// DuoBot
// ------------------------------------------------------------------
// Copright  2005 - Stewart Gray - All Rights Reserved.
//
// BlackGremlin.cpp: implementation of the CBlackGremlin class.
//
//////////////////////////////////////////////////////////////////////

#include "BlackGremlin.h"

const int MAX_FRAME_WAIT = 50;
const int DEF_HIT_POINTS = 1;
const float FLOOR_STOP_Y = 380.0;
const float DEAD_FLOOR_Y = 480;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBlackGremlin::CBlackGremlin()
{
	bDying = false;
	bDead = true;
	hitPoints = DEF_HIT_POINTS;
	spriteSheet = NULL;
	WALK_LEFT = new std::string("WALK_LEFT");
	WALK_RIGHT = new std::string("WALK_RIGHT");
}

CBlackGremlin::~CBlackGremlin()
{
	delete WALK_LEFT;
	delete WALK_RIGHT;
}

void CBlackGremlin::Init(SDL_Surface * spriteSheet)
{

	bDead = true;
	bDying = false;
	setSpriteSheet(spriteSheet);
	CNamedRectList* animation;
	clearAnimations();

	//This is all hard coded junk.

	//Walking facing Left
	animation = new CNamedRectList();
	animation->setName(*WALK_LEFT);
	
	animation->addRect(0, 0, 43, 82);
	animation->addRect(43, 0, 34, 82);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*WALK_RIGHT);

	animation->addRect(77, 0, 45, 82);
	animation->addRect(122, 0, 34, 82);

	addAnimation(animation);

	setAnimation(*WALK_RIGHT);

}

void CBlackGremlin::Update(Uint32 deltaMilliSeconds, float minX, float maxX)
{
	std::string* currentAnim;

	currentAnim = currentAnimation->getName();

	iFrameWait += deltaMilliSeconds;

	if (iFrameWait >= MAX_FRAME_WAIT)
	{
		iFrameWait = 0;
		nextFrame();
	}

	if (bDying)
	{
		footY += 3.0;
		if (footY - currentAnimation->Item(iFrame)->h > DEAD_FLOOR_Y)
		{
			bDying = false;
			bDead = true;
		}
	}
	else
	{
		if (footY < FLOOR_STOP_Y)
		{
			footY += 3.0;
			if (footY > FLOOR_STOP_Y)
			{
				footY = FLOOR_STOP_Y;
			}
		}
		else
		{
			// Right now, all he does is pace.
			if (currentAnim->compare(*WALK_RIGHT) == 0)
			{
				footX += 3.0;
				if (footX > maxX)
				{
					footX = maxX;
					setAnimation(*WALK_LEFT);
				}
			}
			else if (currentAnim->compare(*WALK_LEFT) == 0)
			{
				footX -= 3.0;
				if (footX < minX)
				{
					footX = minX;
					setAnimation(*WALK_RIGHT);
				}
			}
		}
	}
}

hitType CBlackGremlin::CollisionDetect(class CZap * oZap)
{
	float centerX;
	float centerY;
	float dist;
	float distFound;
	float zapX;
	float zapY;

	hitType retVal = HIT_NONE;

	if ((!bDead) && (!bDying))
	{
		if (!oZap->getDead())
		{
			dist = oZap->getRadius();
			dist = dist * dist; // Will be comparing the square of the distance;

			zapX = oZap->getCenterX();
			zapY = oZap->getCenterY();

			centerX = footX;
			centerY = footY - (float) ((3 * currentAnimation->Item(iFrame)->h) / 4);

			distFound = ((centerX - zapX) * (centerX - zapX)) + 
						((centerY - zapY) * (centerY - zapY));

			if (distFound <= dist)
			{
				// Hit.
				if (oZap->getIsBlack())
				{
					hitPoints++;
					retVal = HIT_HELP;
				}
				else
				{
					hitPoints--;
					retVal = HIT_HURT;
					if (hitPoints <= 0)
					{
						bDying = true;
					}
				}

			}
		}
	}

	return retVal;
}

void CBlackGremlin::setHitPoints(int value)
{
	hitPoints = value;
}

int CBlackGremlin::getHitPoints()
{
	return hitPoints;
}

bool CBlackGremlin::getIsDead()
{
	return bDead;
}

void CBlackGremlin::setIsDead(bool value)
{
	bDead = value;
}

void CBlackGremlin::getCollisionRect(int * x, int * y, int * w, int * h)
{
	*x = (int)footX - 15;
	*y = (int)footY - (currentAnimation->Item(iFrame)->h) + 3;
	*w = 30;
	*h = currentAnimation->Item(iFrame)->h - 6;
}
