// DuoBot
// ------------------------------------------------------------------
// Copright  2005 - Stewart Gray - All Rights Reserved.
//
// DuoBot.cpp: implementation of the CDuoBot class.
//
//////////////////////////////////////////////////////////////////////

#include "DuoBot.h"

const int MAX_FRAME_WAIT = 45;
const int MAX_SHOOT_WAIT = 80;
const float FLOOR_Y = 382.0;
const int MAX_HIT_POINTS = 10;
const int MAX_INVINCIBLE_COUNT = 500;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDuoBot::CDuoBot()
{
	invincibleCount = 0;
	bDead = true;
	bWeaponBlack = true;
	hitPoints = MAX_HIT_POINTS;
	WALK_LEFT = new std::string("WALK_LEFT");
	WALK_RIGHT = new std::string("WALK_RIGHT");
	SHOOT_LEFT = new std::string("SHOOT_LEFT");
	SHOOT_RIGHT = new std::string("SHOOT_RIGHT");
	IDLE_LEFT = new std::string("IDLE_LEFT");
	IDLE_RIGHT = new std::string("IDLE_RIGHT");
	JUMP_SHOOT_LEFT = new std::string("JUMP_SHOOT_LEFT");	
	JUMP_RIGHT = new std::string("JUMP_RIGHT");	
	JUMP_SHOOT_RIGHT = new std::string("JUMP_SHOOT_RIGHT");	
	JUMP_LEFT = new std::string("JUMP_LEFT");	

	bFaceLeft = false;
	iFrame = 0;
	iShootWait = 0;
	iFrameWait = 0;
}

CDuoBot::~CDuoBot()
{
	delete WALK_LEFT;
	delete WALK_RIGHT;
	delete SHOOT_LEFT;
	delete SHOOT_RIGHT;
	delete IDLE_LEFT;
	delete IDLE_RIGHT;
	delete JUMP_SHOOT_LEFT;	
	delete JUMP_RIGHT;	
	delete JUMP_SHOOT_RIGHT;	
	delete JUMP_LEFT;	
}

void CDuoBot::Init(SDL_Surface * spriteSheet)
{
	hitPoints = MAX_HIT_POINTS;
	setSpriteSheet(spriteSheet);
	CNamedRectList* animation;
	clearAnimations();

	//This is all hard coded junk.

	//Walking facing Left
	animation = new CNamedRectList();
	animation->setName(*WALK_LEFT);
	animation->addRect(0, 0, 74, 120);
	animation->addRect(74, 0, 54, 120);
	animation->addRect(128, 0, 29, 120);
	animation->addRect(157, 0, 54, 120);
	animation->addRect(211, 0, 83, 120);
	animation->addRect(294, 0, 98, 120);

	//Now put in the reverse frames
	animation->addRect(294, 0, 98, 120);
	animation->addRect(211, 0, 83, 120);
	animation->addRect(157, 0, 54, 120);
	animation->addRect(128, 0, 29, 120);
	animation->addRect(74, 0, 54, 120);
	animation->addRect(0, 0, 74, 120);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*WALK_RIGHT);

	animation->addRect(0, 120, 81, 120);
	animation->addRect(81, 120, 60, 120);
	animation->addRect(141, 120, 33, 120);
	animation->addRect(174, 120, 54, 120);
	animation->addRect(228, 120, 79, 120);
	animation->addRect(307, 120, 90, 120);

	// Reverse the frames to complete the animation.
	animation->addRect(307, 120, 90, 120);
	animation->addRect(228, 120, 79, 120);
	animation->addRect(174, 120, 54, 120);
	animation->addRect(141, 120, 33, 120);
	animation->addRect(81, 120, 60, 120);
	animation->addRect(0, 120, 81, 120);

	addAnimation(animation);


	animation = new CNamedRectList();
	animation->setName(*SHOOT_LEFT);

	animation->addRect(0, 480, 47, 120);
	animation->addRect(47, 480, 59, 120);
	animation->addRect(106, 480, 74, 120);
	animation->addRect(180, 480, 75, 120);

	// Shake a 2nd time
	animation->addRect(106, 480, 74, 120);
	animation->addRect(180, 480, 75, 120);

	// Reverse the sequence.
	animation->addRect(180, 480, 75, 120);
	animation->addRect(106, 480, 74, 120);
	animation->addRect(180, 480, 75, 120);
	animation->addRect(106, 480, 74, 120);
	animation->addRect(47, 480, 59, 120);
	animation->addRect(0, 480, 47, 120);
	
	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*SHOOT_RIGHT);

	animation->addRect(0, 600, 46, 120);
	animation->addRect(46, 600, 61, 120);
	animation->addRect(107, 600, 75, 120);
	animation->addRect(182, 600, 76, 120);

	//Shake a 2nd time.
	animation->addRect(107, 600, 75, 120);
	animation->addRect(182, 600, 76, 120);

	
	// Reverse the sequence to finish.
	animation->addRect(182, 600, 76, 120);
	animation->addRect(107, 600, 75, 120);
	animation->addRect(182, 600, 76, 120);
	animation->addRect(107, 600, 75, 120);
	animation->addRect(46, 600, 61, 120);
	animation->addRect(0, 600, 46, 120);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*JUMP_LEFT);

	//Short animation, double the longevity
	//Maybe next time I will be smart enought to add
	//a longevity to the named rectangle class.
	animation->addRect(0, 240, 45, 120);
	animation->addRect(0, 240, 45, 120);

	animation->addRect(45, 240, 53, 120);
	animation->addRect(45, 240, 53, 120);

	animation->addRect(98, 240, 89, 120);
	animation->addRect(98, 240, 89, 120);

	animation->addRect(187,240, 92, 120);
	animation->addRect(187,240, 92, 120);

	// Reverse it too.
	animation->addRect(187,240, 92, 120);
	animation->addRect(187,240, 92, 120);

	animation->addRect(98, 240, 89, 120);
	animation->addRect(98, 240, 89, 120);

	animation->addRect(45, 240, 53, 120);
	animation->addRect(45, 240, 53, 120);
	
	animation->addRect(0, 240, 45, 120);
	animation->addRect(0, 240, 45, 120);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*JUMP_SHOOT_LEFT);

	// Same frame 4 times.
	animation->addRect(279,240,95,120);
	animation->addRect(279,240,95,120);
	animation->addRect(279,240,95,120);
	animation->addRect(279,240,95,120);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*JUMP_RIGHT);

	// Yup twice and then reverse twice again.
	animation->addRect(0, 360, 44, 120);
	animation->addRect(0, 360, 44, 120);

	animation->addRect(44,360, 51, 120);
	animation->addRect(44,360, 51, 120);

	animation->addRect(95, 360, 88, 120);
	animation->addRect(95, 360, 88, 120);

	animation->addRect(183, 360, 88, 120);
	animation->addRect(183, 360, 88, 120);


	animation->addRect(183, 360, 88, 120);
	animation->addRect(183, 360, 88, 120);

	animation->addRect(95, 360, 88, 120);
	animation->addRect(95, 360, 88, 120);

	animation->addRect(44,360, 51, 120);
	animation->addRect(44,360, 51, 120);

	animation->addRect(0, 360, 44, 120);
	animation->addRect(0, 360, 44, 120);

	addAnimation(animation);
	
	animation = new CNamedRectList();
	animation->setName(*JUMP_SHOOT_RIGHT);
	
	// Yup 4 times again.
	animation->addRect(271, 360, 95, 120);
	animation->addRect(271, 360, 95, 120);
	animation->addRect(271, 360, 95, 120);
	animation->addRect(271, 360, 95, 120);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*IDLE_LEFT);

	animation->addRect(128, 0, 29, 120);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*IDLE_RIGHT);

	animation->addRect(141, 120, 33, 120);

	addAnimation(animation);


	setAnimation(*IDLE_RIGHT);
	
}

void CDuoBot::faceLeft()
{

	std::string* currentAnim;

	if (!bDead)
	{

		if (!bFaceLeft)
		{
			currentAnim = currentAnimation->getName();
			bFaceLeft = true;
			if ((currentAnim->compare(*WALK_LEFT) == 0) ||
				(currentAnim->compare(*SHOOT_LEFT) == 0) ||
				(currentAnim->compare(*JUMP_SHOOT_LEFT) == 0) ||
				(currentAnim->compare(*JUMP_LEFT) == 0) ||
				(currentAnim->compare(*IDLE_LEFT) == 0))
			{
				// Do nothing already has a left animation.
			}
			else if ((currentAnim->compare(*WALK_RIGHT) == 0) ||
					 (currentAnim->compare(*IDLE_RIGHT) == 0))
			{
				setAnimation(*IDLE_LEFT);
			}
			else if (currentAnim->compare(*SHOOT_RIGHT) == 0)
			{
				setAnimation(*SHOOT_LEFT);
			}
			// Don't allow changing direction in mid-air
			// it lets you jump up forever.
			/*
			else if (currentAnim->compare(*JUMP_SHOOT_RIGHT) == 0)
			{
				setAnimation(*JUMP_SHOOT_LEFT);
			}
			else if (currentAnim->compare(*JUMP_RIGHT) == 0)
			{
				setAnimation(*JUMP_LEFT);
			}
			*/

		}
	}
}

void CDuoBot::faceRight()
{
	std::string* currentAnim;

	if (!bDead)
	{
		if (bFaceLeft)
		{
			bFaceLeft = false;
			currentAnim = currentAnimation->getName();

			if ((currentAnim->compare(*WALK_RIGHT) == 0) ||
				(currentAnim->compare(*SHOOT_RIGHT) == 0) ||
				(currentAnim->compare(*JUMP_SHOOT_RIGHT) == 0) ||
				(currentAnim->compare(*JUMP_RIGHT) == 0) ||
				(currentAnim->compare(*IDLE_RIGHT) == 0))
			{
				// Do nothing already has a RIGHT animation.
			}
			else if ((currentAnim->compare(*WALK_LEFT) == 0) ||
					 (currentAnim->compare(*IDLE_LEFT) == 0))
			{
				setAnimation(*IDLE_RIGHT);
			}
			else if (currentAnim->compare(*SHOOT_LEFT) == 0)
			{
				setAnimation(*SHOOT_RIGHT);
			}

			// Don't allow changing direction in mid-air
			// it lets you jump up forever.
			/*
			else if (currentAnim->compare(*JUMP_SHOOT_LEFT) == 0)
			{
				setAnimation(*JUMP_SHOOT_RIGHT);
			}
			else if (currentAnim->compare(*JUMP_LEFT) == 0)
			{
				setAnimation(*JUMP_RIGHT);
			}
			*/
		}
	}
}

void CDuoBot::Update(Uint32 deltaMilliSeconds, bool leftDown, bool rightDown)
{

	bool bJumping;
	bool bStopped;
	bool bWalk;

	if (!bDead)
	{

		std::string* currentAnim;

		currentAnim = currentAnimation->getName();
		
		iFrameWait += deltaMilliSeconds;
		if (iFrameWait >= MAX_FRAME_WAIT)
		{
			nextFrame();
		}

		if (invincibleCount > 0)
		{
			invincibleCount -= deltaMilliSeconds;
			if (invincibleCount < 0)
				invincibleCount = 0;
		}

		if (iShootWait < MAX_SHOOT_WAIT)
		{
			iShootWait += deltaMilliSeconds;
		}

		bStopped = (currentAnim->compare(*IDLE_LEFT) == 0) || (currentAnim->compare(*IDLE_RIGHT) == 0);
		if (bStopped)
		{
			if (footY == FLOOR_Y)
			{
				if (bFaceLeft)
				{
					if (leftDown)
						setAnimation(*WALK_LEFT);
				}
				else
				{
					if (rightDown)
						setAnimation(*WALK_RIGHT);
				}
			}
		}

		bWalk = ((currentAnim->compare(*WALK_LEFT) == 0) || (currentAnim->compare(*WALK_RIGHT) == 0));

		if (bWalk && !(leftDown || rightDown))
		{
			if (bFaceLeft)
				setAnimation(*IDLE_LEFT);
			else
				setAnimation(*IDLE_RIGHT);
			
			bWalk = false;
		}

		bJumping = (currentAnim->compare(*JUMP_LEFT) == 0) || 
				   (currentAnim->compare(*JUMP_RIGHT) == 0) ||
				   (currentAnim->compare(*JUMP_SHOOT_LEFT) == 0) || 
				   (currentAnim->compare(*JUMP_SHOOT_RIGHT) == 0);

		if (bJumping)
		{
			footY -= 3.0;
		}
		else
		{
			footY += 3.0;
			if (footY > FLOOR_Y)
			{
				footY = FLOOR_Y;
			}
		}

		if (bWalk)
		{
			if (bFaceLeft)
			{
				footX -= 3.0;
			}
			else
			{
				footX += 3.0;
			}

			// Stay on screen, dumb hack for now.

			if (footX < 0)
			{
				footX = 0;
			}
			if (footX > 640)
			{
				footX = 640;
			}

		}
	}
}

void CDuoBot::animationLoop()
{
	// Called when an animation loops;
	std::string* currName;

	currName = currentAnimation->getName();

	if (currName->compare(*SHOOT_LEFT)==0)
	{
		setAnimation(*IDLE_LEFT);
	}
	else if (currName->compare(*SHOOT_RIGHT)==0)
	{
		setAnimation(*IDLE_RIGHT);
	}
	else if (currName->compare(*JUMP_SHOOT_LEFT)==0)
	{
		setAnimation(*IDLE_LEFT);
	}
	else if (currName->compare(*JUMP_SHOOT_RIGHT)==0)
	{
		setAnimation(*IDLE_RIGHT);
	}
	else if (currName->compare(*JUMP_LEFT)==0)
	{
		setAnimation(*IDLE_LEFT);
	}
	else if (currName->compare(*JUMP_RIGHT)==0)
	{
		setAnimation(*IDLE_RIGHT);
	}
}

void CDuoBot::shootGun(shootInfo* ShootInfo)
{
	std::string* animName;

	if (!bDead)
	{
		animName = currentAnimation->getName();
		
		if (iShootWait >= MAX_SHOOT_WAIT)
		{	
			ShootInfo->bShoot = true;

			if (bFaceLeft)
			{
				ShootInfo->dir = -1;
			}
			else
			{
				ShootInfo->dir = 1;
			}

			ShootInfo->isBlack = bWeaponBlack;

			if (bFaceLeft)
			{
				
				ShootInfo->x = footX - (currentAnimation->Item(iFrame)->w / 2) + 3;
				ShootInfo->y = footY - (currentAnimation->Item(iFrame)->h / 2);
			}
			else
			{
				ShootInfo->x = footX + (currentAnimation->Item(iFrame)->w / 2) - 3;
				ShootInfo->y = footY - (currentAnimation->Item(iFrame)->h / 2);
			}
			iShootWait = 0;
		}
		else
		{
			ShootInfo->bShoot = false;
		}
		

		if ((animName->compare(*SHOOT_LEFT) != 0) && 
			(animName->compare(*JUMP_SHOOT_LEFT) != 0) && 
			(animName->compare(*SHOOT_RIGHT) != 0) && 
			(animName->compare(*JUMP_SHOOT_RIGHT) != 0))
		{
			if (bFaceLeft)
			{

				if (animName->compare(*JUMP_LEFT) == 0)
				{
					setAnimation(*JUMP_SHOOT_LEFT);
				}
				else
				{
					setAnimation(*SHOOT_LEFT);
				}
			}
			else
			{
				if (animName->compare(*JUMP_RIGHT) == 0)
				{
					setAnimation(*JUMP_SHOOT_RIGHT);
				}
				else
				{
					setAnimation(*SHOOT_RIGHT);
				}
			}
		}
	}
}

bool CDuoBot::Jump()
{
	bool bRet;
	std::string* animName;

	bRet = false;

	if (!bDead)
	{
		if (footY == FLOOR_Y)
		{
			animName = currentAnimation->getName();

			if ((animName->compare(*JUMP_LEFT) != 0) && 
				(animName->compare(*JUMP_SHOOT_LEFT) != 0) && 
				(animName->compare(*JUMP_RIGHT) != 0) && 
				(animName->compare(*JUMP_SHOOT_RIGHT) != 0))
			{
				if (bFaceLeft)
				{
					setAnimation(*JUMP_LEFT);
					bRet = true;
				}
				else
				{
					setAnimation(*JUMP_RIGHT);
					bRet = true;
				}
			}
		}
	}

	return bRet;
}

void CDuoBot::toggleWeapon()
{
	bWeaponBlack = !bWeaponBlack;
}

bool CDuoBot::getWeaponBlack()
{
	return bWeaponBlack;
}

void CDuoBot::setWeaponBlack(bool value)
{
	bWeaponBlack = value;
}

int CDuoBot::getHitPoints()
{
	return hitPoints;
}

void CDuoBot::setHitPoints(int value)
{
	if (value >= 0)
	{
		// Don't allow negative hit points
		if (value <= MAX_HIT_POINTS)
		{
			hitPoints = value;
		}
		else
		{
			hitPoints = MAX_HIT_POINTS;
		}
	}
}

hitType CDuoBot::CollisionDetect(int x, int y, int w, int h)
{

	hitType retVal;

	int botX;
	int botY;
	int botW;
	int botH;

	int minX;
	int minY;
	int maxX;
	int maxY;

	retVal = HIT_NONE;

	if ((!bDead) && (invincibleCount == 0))
	{
		botX = (int) footX - 15;
		botY = (int) footY - currentAnimation->Item(iFrame)->h + 3;
		botW = 30;
		botH = currentAnimation->Item(iFrame)->h - 6;

		minX = botX;
		minY = botY;
		maxX = botX + botW;
		maxY = botY + botH;

		if (x < minX)
			minX = x;

		if (x + w > maxX)
			maxX = x + w;

		if (y < minY)
			minY = y;

		if (y + h > maxY)
			maxY = y + h;


		if (((maxX - minX) <= (botW + w)) &&
			((maxY - minY) <= (botH + h)))
		{
			//Hit.
			retVal = HIT_HURT;
			hitPoints--;
			if (hitPoints <= 0)
			{
				bDead = true;
			}
			else
			{
				// When hit be invincible for a bit.
				invincibleCount = MAX_INVINCIBLE_COUNT;
			}

		}

	}

	return retVal;

}

bool CDuoBot::getIsDead()
{
	return bDead;
}

void CDuoBot::setIsDead(bool value)
{
	bDead = value;
}
