// DuoBot
// ------------------------------------------------------------------
// Copright  2005 - Stewart Gray - All Rights Reserved.
//
// Sprite.cpp: implementation of the CSprite class.
//
//////////////////////////////////////////////////////////////////////

#include "Sprite.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSprite::CSprite()
{
	spriteSheet = NULL;
	animations = new std::vector<class CNamedRectList*>;
	currentAnimation = NULL;
	bVisible = false;
	iFrameWait = 0;
}

CSprite::~CSprite()
{
	animations->erase(animations->begin(), animations->end());
	delete animations;
}

void CSprite::Draw(SDL_Surface * screen, int worldX, int worldY)
{
	bool bSkip = false;
	int destX;
	int destY;
	int srcX;
	int srcY;
	int w;
	int h;
	SDL_Rect* r;

	if (bVisible)
	{
		if (spriteSheet != NULL)
		{
			if (currentAnimation != NULL)
			{
				r = currentAnimation->Item(iFrame);
				srcX = r->x;
				srcY = r->y;
				destX = (int)(footX) - (r->w / 2) - worldX;
				destY = (int)(footY) - r->h - worldY;
				w = r->w;
				h = r->h;

				if (destX + w < 0)
				{
					bSkip = true;
				}
				else if (destX < 0)
				{
					// My code isn't working, let SDL clip it.
					//w = w + destX;
					//srcX = r->x - abs(destX);
				}

				if (destX >= screen->w)
				{
					bSkip = true;
				}
				else if (destX + w >= screen->w)
				{
					w = screen->w - destX;
				}

				if (destY + h < 0)
				{
					bSkip = true;
				}
				else if (destY < 0)
				{
					h = h + destY;
					srcY -= destY;
				}
				else if (destY + h >= screen->h)
				{
					h = screen->h - destY;
				}
				
				if (!bSkip)
				{
					posRect.x = destX;
					posRect.y = destY;
					
					writeRect.x = srcX;
					writeRect.y = srcY;
					writeRect.w = w;
					writeRect.h = h;
					
					SDL_BlitSurface(spriteSheet, &writeRect, screen, &posRect);

				}
			}
		}
	}
}

bool CSprite::getVisible()
{
	return bVisible;
}

void CSprite::setVisible(bool value)
{
	bVisible = value;
}

void CSprite::setAnimation(std::string Name)
{
	bool bFound = false;

	for(std::vector<class CNamedRectList *>::iterator iter = animations->begin(); iter != animations->end(); iter++)
	{
		if ((*iter)->getName()->compare(Name) == 0)
		{
			bFound = true;
			currentAnimation = (*iter);
			break;
		}
	}

	if (bFound) 
	{
		iFrame = 0;
		iFrameWait = 0;
	}
	else
		currentAnimation = NULL;

}

void CSprite::setFrame(int value)
{
	if ((value >= 0) && (value < currentAnimation->getCount()))
	{
		iFrame = value;
	}
	else
	{
		// Call notification function.
		animationLoop();
		iFrame = 0;
	}
}

int CSprite::getFrame()
{
	return iFrame;
}

void CSprite::nextFrame()
{
	setFrame(getFrame() + 1); //Set will take care of wrapping back to zero.
	iFrameWait = 0;
}

void CSprite::setSpriteSheet(SDL_Surface * SpriteSheet)
{
	spriteSheet = SpriteSheet;
}

void CSprite::addAnimation(class CNamedRectList* const Animation)
{
	animations->push_back(Animation);	
}

void CSprite::clearAnimations()
{
	currentAnimation = NULL;

	animations->erase(animations->begin(), animations->end());
	delete animations;

	animations = new std::vector<class CNamedRectList*>;
	iFrame = 0;

}

void CSprite::setX(float value)
{
	footX = value;
}

float CSprite::getX()
{
	return footX;
}

void CSprite::setY(float value)
{
	footY = value;
}

float CSprite::getY()
{
	return footY;
}

void CSprite::animationLoop()
{
	// Do Nothing;
}
