// DuoBot
// ------------------------------------------------------------------
// Copright  2005 - Stewart Gray - All Rights Reserved.
//
// WhiteGhost.cpp: implementation of the CWhiteGhost class.
//
//////////////////////////////////////////////////////////////////////

#include <math.h>
#include "WhiteGhost.h"

const int MAX_FRAME_WAIT = 50;
const double PI = 3.14159;
const double TWO_PI = PI * 2;
const double MIN_INCR = 0.067;
const float DEAD_FLOOR_Y = 480.0;
const float FLOOR_STOP_Y = 350.0;
const int DEF_HIT_POINTS = 1;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWhiteGhost::CWhiteGhost()
{
	bDead = true;
	bDying = false;
	bLanded = false;
	hitPoints = DEF_HIT_POINTS;
	spriteSheet = NULL;
	WALK_LEFT = new std::string("WALK_LEFT");
	WALK_RIGHT = new std::string("WALK_RIGHT");
	radian = 0;
}

CWhiteGhost::~CWhiteGhost()
{
	delete WALK_LEFT;
	delete WALK_RIGHT;
}

void CWhiteGhost::Init(SDL_Surface * spriteSheet)
{
	bDead = true;
	bDying = false;
	bLanded = false;
	setSpriteSheet(spriteSheet);
	CNamedRectList* animation;
	clearAnimations();

	//This is all hard coded junk.

	//Walking facing Left
	animation = new CNamedRectList();
	animation->setName(*WALK_LEFT);
	
	animation->addRect(0, 82, 69, 93);
	animation->addRect(69, 82, 71, 93);

	addAnimation(animation);

	animation = new CNamedRectList();
	animation->setName(*WALK_RIGHT);

	animation->addRect(140, 82, 69, 93);
	animation->addRect(209, 82, 69, 93);

	addAnimation(animation);

	setAnimation(*WALK_RIGHT);
}

void CWhiteGhost::Update(Uint32 deltaMilliSeconds, float minX, float maxX)
{
	std::string* currentAnim;

	radian += MIN_INCR;
	
	while (radian > TWO_PI)
		radian-= TWO_PI;

	iFrameWait += deltaMilliSeconds;

	if (iFrameWait >= MAX_FRAME_WAIT)
	{
		iFrameWait = 0;
		nextFrame();
	}

	if (bDying)
	{
		footY += 3.0;
		if (footY - currentAnimation->Item(iFrame)->h > DEAD_FLOOR_Y)
		{
			bDying = false;
			bDead = true;
		}
	}
	else if (!bLanded)
	{
		footY += 3.0;
		if (footY > FLOOR_STOP_Y)
		{
			footY = FLOOR_STOP_Y;
			bLanded = true;
		}
	}
	else
	{
		currentAnim = currentAnimation->getName();
		setY((double) getY() + (sin(radian) * 1.5));

		// Right now, all he does is pace.
		if (currentAnim->compare(*WALK_RIGHT) == 0)
		{
			footX += 3.0;
			if (footX > maxX)
			{
				footX = maxX;
				setAnimation(*WALK_LEFT);
			}
		}
		else if (currentAnim->compare(*WALK_LEFT) == 0)
		{
			footX -= 3.0;
			if (footX < minX)
			{
				footX = minX;
				setAnimation(*WALK_RIGHT);
			}
		}
	}
}

hitType CWhiteGhost::CollisionDetect(class CZap * oZap)
{
	float centerX;
	float centerY;
	float dist;
	float distFound;
	float zapX;
	float zapY;

	hitType retVal = HIT_NONE;

	if ((!bDead) && (!bDying))
	{
		if (!oZap->getDead())
		{
			dist = oZap->getRadius();
			dist = dist * dist; // Will be comparing the square of the distance;

			zapX = oZap->getCenterX();
			zapY = oZap->getCenterY();

			centerX = footX;
			centerY = footY - (float) (currentAnimation->Item(iFrame)->h / 2);

			distFound = ((centerX - zapX) * (centerX - zapX)) + 
						((centerY - zapY) * (centerY - zapY));

			if (distFound <= dist)
			{
				// Hit.
				if (oZap->getIsBlack())
				{
					hitPoints--;
					retVal = HIT_HURT;
					if (hitPoints <= 0)
					{
						bDying = true;
					}
				}
				else
				{
					hitPoints++;
					retVal = HIT_HELP;
				}

			}
		}
	}

	return retVal;
}

void CWhiteGhost::setHitPoints(int value)
{
	hitPoints = value;
}

int CWhiteGhost::getHitPoints()
{
	return hitPoints;
}

bool CWhiteGhost::getIsDead()
{
	return bDead;
}

void CWhiteGhost::setIsDead(bool value)
{
	bDead = value;
}

void CWhiteGhost::getCollisionRect(int * x, int * y, int * w, int * h)
{
	*x = (int)footX - 15;
	*y = (int)footY - currentAnimation->Item(iFrame)->h + 3;
	*w = 30;
	*h = currentAnimation->Item(iFrame)->h - 6;
}

bool CWhiteGhost::getLanded()
{
	return bLanded;
}

void CWhiteGhost::setLanded(bool value)
{
	value = bLanded;
}
