// DuoBot
// ------------------------------------------------------------------
// Copright  2005 - Stewart Gray - All Rights Reserved.
//
// See readme.txt for the license you may use the code with and
// instructions on how to play.
// ------------------------------------------------------------------
//

#include <string.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_Mixer.h>

#include "DuoBot.h"
#include "BlackGremlin.h"
#include "WhiteGhost.h"
#include "Zap.h"

enum GAME_STATES
{
	GAME_TITLE = 0,
	GAME_PLAY = 1, 
	GAME_PAUSE = 2,
	GAME_QUIT_PLAY = 3,
	GAME_QUIT_TITLE = 4,
	GAME_NEXTLEVEL = 5,
	GAME_WIN = 6,
	GAME_LOSE = 7
};

int main(int argc, char** argv)
{

	const int LEVEL_COUNT = 2;
	const int MAX_GHOSTS = 100;
	const int MAX_GREMLINS = 100;
	const int MAX_ZAPS = 100;

	const int SCREEN_WIDTH = 640;
	const int SCREEN_HEIGHT = 480;
	const int COLOR_DEPTH = 16;

	const int MAX_FRAME_WAIT = 45;

	bool bAnyKeyPressed = false;
	bool bDontPauseAgain = false;
	bool bRunLine = false;
	bool bPrevAnyKeyPressed = false;
	bool bPrevShiftDown = false;
	bool bShiftDown = false;
		
	bool bFullScreen = true;
	bool bLeftDown = false;
	bool bRightDown = false;
	bool bQuit = false;

	GAME_STATES gameState = GAME_TITLE;

	char* levels[] = {"level1.txt", "level2.txt"};

	int levelNum = 0;
	int numKeys = 0;

	FILE* fp = NULL;

	shootInfo ShootInfo;

	Uint32 colorBlack = 0;
	Uint32 deltaTime = 0;
	Uint32 gameTime = 0;

	float fDeltaTime = 0.0;
	
	int freeGhostIDX = 0;
	int freeGremlinIDX = 0;

	class CDuoBot* oDuoBot;
	class CBlackGremlin* oBlackGremlin[MAX_GREMLINS];
	class CWhiteGhost* oWhiteGhost[MAX_GHOSTS];
	class CZap* zaps[MAX_ZAPS];

	int colX;
	int colY;
	int colW;
	int colH;

	int i;
	int j;

	hitType hitRetVal;

	Uint8* key;

	int targetX;
	int targetY;
	Uint32 targetTime;
	char targetType;

	Mix_Chunk* gameOverSound;
	Mix_Chunk* haHaSound[2];
	Mix_Chunk* hitSound;
	Mix_Chunk* jumpSound;
	Mix_Chunk* levelCompleteSound;
	Mix_Chunk* ouchSound[2];
	Mix_Chunk* titleSound;
	Mix_Chunk* youWinSound;
	Mix_Chunk* zapSound[2];

	SDL_Event event;

	SDL_Surface* backgroundPic = NULL;
	SDL_Surface* blackZap = NULL;
	SDL_Surface* duoBotSprite = NULL;
	SDL_Surface* enemySprite = NULL;
	SDL_Surface* losePopup = NULL;
	SDL_Surface* levelCompletePopup = NULL;
	SDL_Surface* pausePopup = NULL;
	SDL_Surface* quitPopup = NULL;
	SDL_Surface* temp = NULL;
	SDL_Surface* titlePic = NULL;
	SDL_Surface* screen = NULL;
	SDL_Surface* weaponHUD = NULL;
	SDL_Surface* whiteZap = NULL;
	SDL_Surface* winPopup = NULL;

	SDL_Rect centerRect;
	SDL_Rect darkRect;
	SDL_Rect hudPos;
	SDL_Rect lightRect;
		
	Uint32 prevTime;
	Uint32 currTime;


	bFullScreen = true;
	for (i=0;i<argc;i++)
	{
		if (strcmp("win" , argv[i])==0)
			bFullScreen = false;
	}

	/* Initialize the SDL library */
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0)
	{
		fprintf (stderr, "Could not initialize SDL:  %s\n", SDL_GetError());
		exit(1);
	}

	// Clean up on exit.
	atexit(SDL_Quit);

    // Initialize the display in a 640x480 16-bit mode,
    if (bFullScreen)
		screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, COLOR_DEPTH, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN); 
	else
		screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, COLOR_DEPTH, SDL_HWSURFACE | SDL_DOUBLEBUF); 

	if (screen == NULL)
	{
		fprintf(stderr, "Couldn't set %dx%x%d video mode: %s\n", SCREEN_WIDTH, SCREEN_HEIGHT, COLOR_DEPTH, SDL_GetError());
		exit(1);
	}

	temp = SDL_LoadBMP("data/DuoBot_sprite.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "DuoBot_sprite.bmp", SDL_GetError());
        return -1;
    }

    /* Set the transparent color */
	SDL_SetColorKey(temp,SDL_SRCCOLORKEY, SDL_MapRGB(temp->format,0,255,0));
	duoBotSprite = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	temp = SDL_LoadBMP("data/Enemy_sprite.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "Enemy_sprite.bmp", SDL_GetError());
        return -1;
    }

    /* Set the transparent color */
	SDL_SetColorKey(temp,SDL_SRCCOLORKEY, SDL_MapRGB(temp->format,0,255,0));
	enemySprite = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	temp = SDL_LoadBMP("data/BlackZap.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "BlackZap.bmp", SDL_GetError());
        return -1;
    }

    /* Set the transparent color */
	SDL_SetColorKey(temp,SDL_SRCCOLORKEY, SDL_MapRGB(temp->format,0,255,0));
	blackZap = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	temp = SDL_LoadBMP("data/WhiteZap.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "WhiteZap.bmp", SDL_GetError());
        return -1;
    }

    /* Set the transparent color */
	SDL_SetColorKey(temp,SDL_SRCCOLORKEY, SDL_MapRGB(temp->format,0,255,0));
	whiteZap = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	temp = SDL_LoadBMP("data/WeaponHUD.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "WeaponHUD.bmp", SDL_GetError());
        return -1;
    }

    /* Set the transparent color */
	SDL_SetColorKey(temp,SDL_SRCCOLORKEY, SDL_MapRGB(temp->format,0,255,0));
	weaponHUD = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	// ---
	temp = SDL_LoadBMP("data/background.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "background.bmp", SDL_GetError());
        return -1;
    }

	backgroundPic = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	// ---
	temp = SDL_LoadBMP("data/LevelComplete.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "LevelComplete.bmp", SDL_GetError());
        return -1;
    }

	levelCompletePopup = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	// ---
	temp = SDL_LoadBMP("data/Lose.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "Lose.bmp", SDL_GetError());
        return -1;
    }

	losePopup = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	// ---
	temp = SDL_LoadBMP("data/Pause.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "Pause.bmp", SDL_GetError());
        return -1;
    }

	pausePopup = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	// ---
	temp = SDL_LoadBMP("data/Quit.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "Quit.bmp", SDL_GetError());
        return -1;
    }

	quitPopup = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	// ---
	temp = SDL_LoadBMP("data/Win.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "Win.bmp", SDL_GetError());
        return -1;
    }

	winPopup = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	// ---
	temp = SDL_LoadBMP("data/Title.bmp");
    if (temp == NULL) {
        fprintf(stderr, "Couldn't load %s: %s\n", "Title.bmp", SDL_GetError());
        return -1;
    }

	titlePic = SDL_DisplayFormat(temp);
	SDL_FreeSurface(temp);
	temp = NULL;

	colorBlack = SDL_MapRGB(duoBotSprite->format, 0, 0, 0);

	//Start up SDL_Mixer
	if (Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, MIX_DEFAULT_FORMAT, 1, 1024)==-1)
	{
		fprintf(stderr, "Couldn't start up SDL_Mixer:  %s\n", Mix_GetError());
		return -1;
	}

	gameOverSound = Mix_LoadWAV("data\\GameOver.wav");
	if (!gameOverSound)
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\GameOver.wav", Mix_GetError());
		return -1;
	}

	haHaSound[0] = Mix_LoadWAV("data\\haha.wav");
	if (!haHaSound[0])
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\haha.wav", Mix_GetError());
		return -1;
	}

	haHaSound[1] = Mix_LoadWAV("data\\haha2.wav");
	if (!haHaSound[1])
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\haha2.wav", Mix_GetError());
		return -1;
	}

	hitSound = Mix_LoadWAV("data\\hit.wav");
	if (!hitSound)
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\hit.wav", Mix_GetError());
		return -1;
	}

	jumpSound = Mix_LoadWAV("data\\jump.wav");
	if (!jumpSound)
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\jump.wav", Mix_GetError());
		return -1;
	}

	levelCompleteSound = Mix_LoadWAV("data\\LevelComplete.wav");
	if (!levelCompleteSound)
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\LevelComplete.wav", Mix_GetError());
		return -1;
	}

	ouchSound[0] = Mix_LoadWAV("data\\ouch.wav");
	if (!ouchSound[0])
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\ouch.wav", Mix_GetError());
		return -1;
	}

	ouchSound[1] = Mix_LoadWAV("data\\ouch2.wav");
	if (!ouchSound[1])
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\ouch2.wav", Mix_GetError());
		return -1;
	}

	titleSound = Mix_LoadWAV("data\\title.wav");
	if (!titleSound)
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\title.wav", Mix_GetError());
		return -1;
	}

	youWinSound = Mix_LoadWAV("data\\YouWin.wav");
	if (!youWinSound)
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\YouWin.wav", Mix_GetError());
		return -1;
	}

	
	zapSound[0] = Mix_LoadWAV("data\\darkZap.wav");
	if (!zapSound[0])
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\darkZap.wav", Mix_GetError());
		return -1;
	}

	zapSound[1] = Mix_LoadWAV("data\\lightZap.wav");
	if (!zapSound[1])
	{
		fprintf(stderr, "Couldn't load %s:  %s\n", "data\\lightZap.wav", Mix_GetError());
		return -1;
	}


	currTime = SDL_GetTicks();
	prevTime = currTime;
	if (prevTime < 0)
		prevTime = 0;

	bQuit = false;

	lightRect.x = 0;
	lightRect.y = 0;
	lightRect.w = weaponHUD->w;
	lightRect.h = weaponHUD->h / 2;

	darkRect.x = 0;
	darkRect.y = lightRect.h;
	darkRect.w = weaponHUD->w;
	darkRect.h = weaponHUD->h / 2;

	hudPos.x = 5;
	hudPos.y = 5;

	oDuoBot = new CDuoBot();

	for (i=0; i < MAX_GREMLINS; i++)
	{
		oBlackGremlin[i] = new CBlackGremlin();
	}

	for (i=0; i < MAX_GHOSTS; i++)
	{
		oWhiteGhost[i] = new CWhiteGhost();
	}

	for (i=0; i < MAX_ZAPS; i++)
	{
		zaps[i] = new CZap();
	}

	Mix_AllocateChannels(16);
	Mix_PlayChannel(-1,titleSound, 0); 

	while (!bQuit)
	{

		prevTime = currTime;
		currTime = SDL_GetTicks();
		deltaTime = currTime - prevTime;

		if (deltaTime > 500)
			deltaTime = 500;

		if (gameState == GAME_PLAY)
		{
			if (!bRunLine)
			{
				// Run the line.
				if (targetTime <= gameTime)
				{
					if (targetType == 'B')
					{
						freeGremlinIDX = -1;
						for (i=0;i<MAX_GREMLINS;i++)
						{
							if (oBlackGremlin[i]->getIsDead())
							{
								freeGremlinIDX = i;
								break;
							}
						}
						if (freeGremlinIDX >= 0)
						{
							// Run it
							oBlackGremlin[freeGremlinIDX]->Init(enemySprite);
							oBlackGremlin[freeGremlinIDX]->setX(targetX);
							oBlackGremlin[freeGremlinIDX]->setY(targetY);
							oBlackGremlin[freeGremlinIDX]->setVisible(true);
							oBlackGremlin[freeGremlinIDX]->setIsDead(false);
							bRunLine = true;
						}
					}
					else if (targetType == 'W')
					{
						for (i=0;i<MAX_GHOSTS;i++)
						{
							if (oWhiteGhost[i]->getIsDead())
							{
								freeGhostIDX = i;
								break;
							}
						}

						if (freeGhostIDX >= 0)
						{
							// Run it
							oWhiteGhost[freeGhostIDX]->Init(enemySprite);
							oWhiteGhost[freeGhostIDX]->setX(targetX);
							oWhiteGhost[freeGhostIDX]->setY(targetY);
							oWhiteGhost[freeGhostIDX]->setVisible(true);
							oWhiteGhost[freeGhostIDX]->setIsDead(false);
							bRunLine = true;
						}
					}
				}
			}

			if (!feof(fp) && bRunLine)
			{
				// Read a new line.
				fscanf(fp,"%d %c %d %d\n", &targetTime, &targetType, &targetX, &targetY);
				bRunLine = false;
			}

			gameTime += deltaTime;

			oDuoBot->Update(deltaTime, bLeftDown, bRightDown);
			for(i=0;i< MAX_GREMLINS;i++)
			{
				if (!oBlackGremlin[i]->getIsDead())
				{
					oBlackGremlin[i]->Update(deltaTime, 0.0, (float)(SCREEN_WIDTH - 1));
				}
			}

			for(i=0;i<MAX_GHOSTS;i++)
			{
				if (!oWhiteGhost[i]->getIsDead())
				{
					oWhiteGhost[i]->Update(deltaTime, 0.0, (float) (SCREEN_WIDTH - 1));
				}
			}

			if (!oDuoBot->getIsDead())
			{
				for (i=0;i<MAX_GREMLINS;i++)
				{
					if (!oBlackGremlin[i]->getIsDead())
					{
						oBlackGremlin[i]->getCollisionRect(&colX, &colY, &colW, &colH);
						hitRetVal = oDuoBot->CollisionDetect(colX, colY, colW, colH);

						if (hitRetVal == HIT_HURT)
						{
							Mix_PlayChannel(-1, hitSound, 0);
						}
					}
				}

				for (i=0;i<MAX_GHOSTS;i++)
				{
					if (!oWhiteGhost[i]->getIsDead())
					{
						oWhiteGhost[i]->getCollisionRect(&colX, &colY, &colW, &colH);
						hitRetVal = oDuoBot->CollisionDetect(colX, colY, colW, colH);

						if (hitRetVal == HIT_HURT)
						{
							Mix_PlayChannel(-1, hitSound, 0);
						}
					}
				}
			}

			fDeltaTime = (float) deltaTime;
			for(i=0;i<MAX_ZAPS;i++)
			{
				if (!zaps[i]->getDead())
				{
					zaps[i]->Update(fDeltaTime);
					for (j=0; j < MAX_GREMLINS;j++)
					{
						if (!oBlackGremlin[j]->getIsDead())
						{
							hitRetVal = oBlackGremlin[j]->CollisionDetect(zaps[i]);
							if (hitRetVal == HIT_HURT)
							{
								Mix_PlayChannel(-1, ouchSound[0], 0);
								zaps[i]->setDead(true);
							}
							else if (hitRetVal == HIT_HELP)
							{
								Mix_PlayChannel(-1, haHaSound[0], 0);
								zaps[i]->setDead(true);
							}
						}
					}

					for (j=0; j < MAX_GHOSTS; j++)
					{
						if (!oWhiteGhost[j]->getIsDead())
						{
							hitRetVal = oWhiteGhost[j]->CollisionDetect(zaps[i]);
							if (hitRetVal == HIT_HURT)
							{
								Mix_PlayChannel(-1, ouchSound[1], 0);
								zaps[i]->setDead(true);
							}
							else if (hitRetVal == HIT_HELP)
							{
								Mix_PlayChannel(-1, haHaSound[1], 0);
								zaps[i]->setDead(true);
							}
						}
					}
				}
			}

		}

		if ((gameState == GAME_PLAY) ||
			(gameState == GAME_PAUSE) ||
			(gameState == GAME_QUIT_PLAY) || 
			(gameState == GAME_NEXTLEVEL) || 
			(gameState == GAME_WIN) ||
			(gameState == GAME_LOSE))
		{
			SDL_BlitSurface(backgroundPic, NULL, screen, NULL);
			for (i=0; i < MAX_GREMLINS; i++)
			{
				if (!oBlackGremlin[i]->getIsDead())
				{
					oBlackGremlin[i]->Draw(screen, 0, 0);
				}
			}

			for (i=0; i < MAX_GHOSTS; i++)
			{
				if (!oWhiteGhost[i]->getIsDead())
				{
					oWhiteGhost[i]->Draw(screen, 0, 0);
				}
			}

			oDuoBot->Draw(screen, 0, 0);
			for(i=0;i<MAX_ZAPS;i++)
			{
				if (!zaps[i]->getDead())
				{
					zaps[i]->Draw(screen, 0, 0);
				}
			}

			if (oDuoBot->getWeaponBlack())
			{
				SDL_BlitSurface(weaponHUD, &darkRect, screen, &hudPos);
			}
			else
			{
				SDL_BlitSurface(weaponHUD, &lightRect, screen, &hudPos);
			}

			// Draw the pause/etc. pictures
			if (gameState == GAME_PAUSE)
			{
				centerRect.x = (screen->w - pausePopup->w)/2;
				centerRect.y = (screen->h - pausePopup->h)/2;
				SDL_BlitSurface(pausePopup, NULL, screen, &centerRect);
			}
			else if (gameState == GAME_QUIT_PLAY)
			{
				centerRect.x = (screen->w - quitPopup->w)/2;
				centerRect.y = (screen->h - quitPopup->h)/2;
				SDL_BlitSurface(quitPopup, NULL, screen, &centerRect);
			}
			else if (gameState == GAME_WIN)
			{
				centerRect.x = (screen->w - winPopup->w)/2;
				centerRect.y = (screen->h - winPopup->h)/2;
				SDL_BlitSurface(winPopup, NULL, screen, &centerRect);
			}
			else if (gameState == GAME_LOSE)
			{
				centerRect.x = (screen->w - losePopup->w)/2;
				centerRect.y = (screen->h - losePopup->h)/2;
				SDL_BlitSurface(losePopup, NULL, screen, &centerRect);
			}
			else if (gameState == GAME_NEXTLEVEL)
			{
				centerRect.x = (screen->w - levelCompletePopup->w)/2;
				centerRect.y = (screen->h - levelCompletePopup->h)/2;
				SDL_BlitSurface(levelCompletePopup, NULL, screen, &centerRect);
			}
		}
		else if ((gameState == GAME_TITLE) ||
			     (gameState == GAME_QUIT_TITLE))
		{
			// Draw the title screen
			SDL_BlitSurface(titlePic, NULL, screen, NULL);
			if (gameState == GAME_QUIT_TITLE)
			{
				centerRect.x = (screen->w - quitPopup->w)/2;
				centerRect.y = (screen->h - quitPopup->h)/2;
				SDL_BlitSurface(quitPopup, NULL, screen, &centerRect);
			}
		}

		// Flip the surface
		SDL_Flip(screen);


		SDL_PumpEvents();
		key = SDL_GetKeyState(&numKeys);

		bPrevAnyKeyPressed = bAnyKeyPressed;
		bAnyKeyPressed = false;
		for(i = 0; i< numKeys; i++)
		{
			if (key[i])
			{
				bAnyKeyPressed = true;
				break;
			}
		}

		if (key[SDLK_LEFT])
		{
			if (gameState == GAME_PLAY)
			{
				oDuoBot->faceLeft();
			}
		}
		else if (key[SDLK_RIGHT])
		{
			if (gameState == GAME_PLAY)
			{
				oDuoBot->faceRight();
			}
		}

		if (key[SDLK_SPACE])
		{
			if (gameState == GAME_PLAY)
			{
				if (oDuoBot->Jump())
				{
					Mix_PlayChannel(-1, jumpSound, 0);
				}
			}
		}

		if ((key[SDLK_LCTRL]) || (key[SDLK_RCTRL]))
		{
			if (gameState == GAME_PLAY)
			{
				oDuoBot->shootGun(&ShootInfo);
				if (ShootInfo.bShoot)
				{
					// Spawn a bullet
					for(i=0;i<MAX_ZAPS;i++)
					{
						if (zaps[i]->getDead())
						{
							if (ShootInfo.isBlack)
							{
								zaps[i]->Init(blackZap, true, ShootInfo.x, ShootInfo.y, ShootInfo.dir, SCREEN_WIDTH);
								Mix_PlayChannel(-1,zapSound[0], 0); 
							}
							else
							{
								zaps[i]->Init(whiteZap, false, ShootInfo.x, ShootInfo.y, ShootInfo.dir, SCREEN_WIDTH);
								Mix_PlayChannel(-1,zapSound[1], 0); 
							}
							zaps[i]->setVisible(true);

							break;
						}
					}
				}
			}
		}

		bLeftDown = (bool) (key[SDLK_LEFT]);
		bRightDown = (bool) (key[SDLK_RIGHT]);
		bPrevShiftDown = bShiftDown;
		bShiftDown = (key[SDLK_LSHIFT] || key[SDLK_RSHIFT]);

		if (gameState == GAME_NEXTLEVEL)
		{
			if (!bPrevAnyKeyPressed)
			{
				if (key[SDLK_SPACE] || key[SDLK_RETURN] || key[SDLK_ESCAPE])
				{
					// Reset everything
					oDuoBot->Init(duoBotSprite);
					oDuoBot->setX(SCREEN_WIDTH /2);
					oDuoBot->setY(382);
					oDuoBot->setVisible(true);
					oDuoBot->setIsDead(false);

					for (i=0; i < MAX_GREMLINS; i++)
					{
						oBlackGremlin[i]->setIsDead(true);
					}

					for (i=0; i < MAX_GHOSTS; i++)
					{
						oWhiteGhost[i]->setIsDead(true);
					}

					for (i=0; i < MAX_ZAPS; i++)
					{
						zaps[i]->setDead(true);
					}

					if (fp != NULL)
					{
						fclose(fp);
					}

					fp = fopen(levels[levelNum], "rt");
					if (fp == NULL)
					{
						fprintf(stderr, "Couldn't open %s\n", levels[levelNum]);
						return -1;
					}

					bRunLine = true;

					gameTime = 0;

					// Switch states.
					gameState = GAME_PLAY;
				}
			}
		}
		else if (gameState == GAME_PAUSE)
		{
 			if (!bPrevAnyKeyPressed)
			{
				if (key[SDLK_SPACE] || key[SDLK_RETURN] || key[SDLK_p] || key[SDLK_ESCAPE])
				{
					gameState = GAME_PLAY;
					bDontPauseAgain = true;
				}
			}
		}
		else if (gameState == GAME_PLAY)
		{
			if (bShiftDown && !bPrevShiftDown)
			{
				oDuoBot->toggleWeapon();
			}

			if (!bPrevAnyKeyPressed)
			{
				if (key[SDLK_p])
				{
					if (!bDontPauseAgain)
					{
						gameState = GAME_PAUSE;
					}
					
				}

				if (key[SDLK_q] || key[SDLK_ESCAPE])
				{
					gameState = GAME_QUIT_PLAY;
				}
			}

			// check for lose
			if (oDuoBot->getIsDead())
			{
				gameState = GAME_LOSE;
				Mix_PlayChannel(-1, gameOverSound, 0);
			}
			else if (feof(fp) && bRunLine)
			{
				// Check if there are any monsters left alive;
				j = 0;
				for (i=0; i < MAX_GREMLINS; i++)
				{
					if (!oBlackGremlin[i]->getIsDead())
					{
						j = 1;
						break;
					}
				}

				if (j==0)
				{
					for(i=0;i<MAX_GHOSTS; i++)
					{
						if (!oWhiteGhost[i]->getIsDead())
						{
							j = 1;
							break;
						}
					}
				}

				if (j == 0)
				{
					// No ghosts or goblins left
					if (fp != NULL)
					{
						fclose(fp);
						fp = NULL;
					}

					levelNum++;
					if (levelNum >= LEVEL_COUNT)
					{
						gameState = GAME_WIN;
						Mix_PlayChannel(-1, youWinSound, 0);
					}
					else
					{
						gameState = GAME_NEXTLEVEL;
						Mix_PlayChannel(-1, levelCompleteSound, 0);
					}
				}
			}
		}
		else if (gameState == GAME_TITLE)
		{
 			if (!bPrevAnyKeyPressed)
			{
				if (key[SDLK_SPACE] || key[SDLK_RETURN])
				{
					// Reset everything
					oDuoBot->Init(duoBotSprite);
					oDuoBot->setX(SCREEN_WIDTH /2);
					oDuoBot->setY(382);
					oDuoBot->setVisible(true);
					oDuoBot->setIsDead(false);

					for (i=0; i < MAX_GREMLINS; i++)
					{
						oBlackGremlin[i]->setIsDead(true);
					}

					for (i=0; i < MAX_GHOSTS; i++)
					{
						oWhiteGhost[i]->setIsDead(true);
					}

					for (i=0; i < MAX_ZAPS; i++)
					{
						zaps[i]->setDead(true);
					}

					if (fp != NULL)
					{
						fclose(fp);
					}

					levelNum = 0;

					fp = fopen(levels[levelNum], "rt");
					if (fp == NULL)
					{
						fprintf(stderr, "Couldn't open %s\n", levels[levelNum]);
						return -1;
					}

					bRunLine = true;

					gameTime = 0;

					// Switch states.
					gameState = GAME_PLAY;
				}
				else if (key[SDLK_ESCAPE] || key[SDLK_q])
				{
					gameState = GAME_QUIT_TITLE;
				}
			}
		}
		else if (gameState == GAME_QUIT_PLAY)
		{
			if (!bPrevAnyKeyPressed)
			{
				if (key[SDLK_ESCAPE] || key[SDLK_n])
				{
					gameState = GAME_PLAY;
				}
				else if (key[SDLK_y])
				{
					if (fp != NULL)
					{
						// The rest can be reset if we come back.
						fclose(fp);
						fp = NULL;
					}
					gameState = GAME_TITLE;
				}
			}
		}
		else if (gameState == GAME_QUIT_TITLE)
		{
			if (!bPrevAnyKeyPressed)
			{
				if (key[SDLK_ESCAPE] || key[SDLK_n])
				{
					gameState = GAME_TITLE;
				}
				else if (key[SDLK_y])
				{
					if (fp != NULL)
					{
						// The rest can be reset if we come back.
						fclose(fp);
						fp = NULL;
					}
					gameState = GAME_TITLE;
					bQuit = true;
				}
			}
		}
		else if ((gameState == GAME_LOSE) || (gameState == GAME_WIN))
		{
			if (!bPrevAnyKeyPressed)
			{
				if (key[SDLK_SPACE] || key[SDLK_RETURN] || key[SDLK_ESCAPE])
				{
					gameState = GAME_TITLE;
				}
			}
		}

		bDontPauseAgain = false;

		// I want the [X] button on the window to work.
		// It dumps you all the way out without question.
		while (SDL_PollEvent(&event))
		{
			if (event.type == SDL_QUIT)
			{
				bQuit = true;
			}
		}			
	}

	//If playing sounds, stop so we can exit.
	if (Mix_Playing(-1)>0)
		Mix_HaltChannel(-1); 

	Mix_FreeChunk(gameOverSound);
	gameOverSound = NULL;
	
	Mix_FreeChunk(haHaSound[0]);
	haHaSound[0] = NULL;
	
	Mix_FreeChunk(haHaSound[1]);
	haHaSound[1] = NULL;

	Mix_FreeChunk(hitSound);
	hitSound = NULL;

	Mix_FreeChunk(jumpSound);
	jumpSound = NULL;

	Mix_FreeChunk(levelCompleteSound);
	levelCompleteSound = NULL;
	
	Mix_FreeChunk(ouchSound[0]);
	ouchSound[0] = NULL;
	
	Mix_FreeChunk(ouchSound[1]);
	ouchSound[1] = NULL;
	
	Mix_FreeChunk(titleSound);
	titleSound = NULL;
	
	Mix_FreeChunk(youWinSound);
	youWinSound = NULL;
	
	Mix_FreeChunk(zapSound[0]);
	zapSound[0] = NULL;
	
	Mix_FreeChunk(zapSound[1]);
	zapSound[1] = NULL;
		
	delete oDuoBot;

	for (i=0; i < MAX_GREMLINS; i++)
	{
		delete oBlackGremlin[i];
	}

	for (i=0; i < MAX_GHOSTS; i++)
	{
		delete oWhiteGhost[i];
	}

	for (i=0; i < MAX_ZAPS; i++)
	{
		delete zaps[i];
	}

	if (fp != NULL)
	{
		fclose(fp);
		fp = NULL;
	}

	SDL_FreeSurface(blackZap);
	SDL_FreeSurface(duoBotSprite);
	SDL_FreeSurface(enemySprite);
	SDL_FreeSurface(backgroundPic);
	SDL_FreeSurface(levelCompletePopup);
	SDL_FreeSurface(losePopup);	
	SDL_FreeSurface(pausePopup);
	SDL_FreeSurface(quitPopup);
	SDL_FreeSurface(titlePic);
	SDL_FreeSurface(weaponHUD);
	SDL_FreeSurface(whiteZap);
	SDL_FreeSurface(winPopup);
	
	return(0);
}